;this program shows how to use your graphics in an ti86 asm-program

#include "asm86.h"
#include "ti86asm.inc"

.org _asm_exec_ram

 call _clrLCD		;erase the screen
 call bw_spr		;show a b&w sprite
 call gr_spr		;show a greyscale sprite
 call bw_pic		;display a b&w picture not compressed
 call bw_pic_rle	;display a b&w picture with rle
 call gr_pic		;display a greyscale picture not compressed
 call gr_pic_rle	;display a greyscale picture with rle
 ret			;quit program

bw_spr:
 call _homeup
 ld hl,bw_spr_str
 call _puts
 ld a,28
 ld e,a
 ld a,60
 ld hl,bw_man
 call bwPutSprite
 call _getkey
 call _clrLCD
 ret
bw_spr_str:  .db "b&w sprite:",0

gr_spr:
 call InitGrey
 call ClearScreen
 call _homeup
 ld hl,gr_spr_str
 call _puts
 ld b,60
 ld c,28
 ld hl,gr_block
 call grPutSprite
 call _getkey
 call DeinitGrey
 call _clrLCD
 ret
gr_spr_str:  .db "grey sprite:",0

bw_pic:
 ld de,Picture
 ld hl,$fc00
 ld a,64
 call LoadPic
 call _getkey
 call _clrLCD
 ret

bw_pic_rle:
 ld hl,rlePicture
 ld de,$fc00
 call DispRLE
 call _getkey
 call _clrLCD
 ret

gr_pic:
 call InitGrey		;enable greyscale
 call ClearScreen
 ld de,Light
 ld hl,$f000
 ld a,64
 call LoadPic
 ld de,Dark
 ld hl,$fc00
 ld a,64
 call LoadPic
 call _getkey
 call _clrLCD
 call DeinitGrey
 ret

gr_pic_rle:
 call InitGrey		;enable greyscale
 call ClearScreen
 ld hl,rleLight		;this is the light layer of the picture
 ld de,$f000		;and we put it to $f000 (first video ram)
 call DispRLE		;decode rle-compressed data to 'de'
 ld hl,rleDark		;same with darklayer
 ld de,$fc00		;dark goes to $fc00
 call DispRLE		;decode...
 call _getkey		;wait for keypress
 call _clrLCD
 call DeinitGrey
 ret


;===========================================================
; RLE picture displayer v1.1
; Decodes a RLE picture made by RLE2PIC
;
; written by David Phillips <electrum@tfs.net>
; started: 8/19/98
; last update: 11/5/98
;
; input: HL = RLE encoded picture, DE = where to display
; output: 1024 byte decoded picture
; destroys: AF, BC, DE, HL
; current size: 32 bytes
;===========================================================
DispRLE:
 ld bc,1024			; we need to copy 
DispRLEL:
 ld a,(hl)			; get the next byte
 cp $91				; is it a run?
 jr z,DispRLERun	; then we need to decode the run
 ldi				; copy the byte, and update counters
DispRLEC:
 ld a,b				; check the low byte and
 or c				; the high byte for 0
 jr nz,DispRLEL		; if not, then we're not done either
 ret				; if it's zero, we're done
DispRLERun:
 inc hl				; move to the run value
 ld a,(hl)			; get the run value
 inc hl				; move to the run count
 push hl			; save source pointer
 ld h,(hl)			; get the run count
 ex de,hl			; swap source and destination pointers
DispRLERunL:
 ld (hl),a			; copy the byte
 inc hl				; increase destination pointer
 dec bc				; decrease byte count
 dec d				; decrease run count
 jr nz,DispRLERunL	; if we're not done, then loop
 ex de,hl			; swap pointers back
 pop hl				; recover source pointer
 inc hl				; advance the source pointer
 jr DispRLEC		; check to see if we should loop

;=========== Routine for displaying a not compressed picture ============
; hl: video ram
; de: picture data
; a:  picture height (width has to be 64)

LoadPic:
    ld b,a
pNextLine:
    push bc
    ld b,16			; there are 16 bytes in each line
pNextByte:
    ld a,(de)
    ld (hl),a 			; get byte and copy to vid mem
    inc de			; increment our de
    inc hl			; inc out hl
    djnz pNextByte
    pop bc
    djnz pNextLine
    ret


;=================== Greyscalemode routines made by SCaBBy ===================
InitGrey:
	call ClearGreyBuffer
	ld hl,IHandler
	ld de,0BCBCh ;0D2FEh
	ld bc,IHandlerEnd-IHandler
	ldir

	ld hl,0BD00h
	ld de,0BD01h
	ld bc,256
	ld (hl),0BCh
	ldir
	ld a,0BDh
	ld i,a
	im 2
	ret

ClearScreen:
	call _clrLCD
ClearGreyBuffer:
	ld hl,GreyBuffer
	ld de,GreyBuffer+1
	ld bc,3FFh
	ld (hl),0
	ldir
	ret

DeinitGrey:
	im 1
	ld a,3Ch
	out (0),a
	ret

IHandler:
	exx
	ex af,af'
	ld hl,Timer
	inc (hl)
	in a,(3)
	bit 1,a
	jr z,notvbl
	ld hl,GreyCounter
	inc (hl)
	ld a,(hl)
	cp 3
	ld a,3Ch
	jr nz,notpage1
	ld (hl),0
	ld a,(GreyBuffer/256)-0C0h
notpage1:
	out (0),a
notvbl:
	jp 3Ah
IHandlerEnd:

GreyCounter	.db 0

Timer		=0BCFFh
GreyBuffer	=0F000h


;=================== Black&White Putsprite routine ===================
;------------------------------------------------------
;|This Routine was converted by AVD from Movax's 83
;|Special thanks to ScaBBy for making it XOR Routine!
;------------------------------------------------------
;hl = image
;a = x coord
;e = y coord

bwPutSprite:
       push    hl              ; Save sprite address

;   Calculate the address in graphbuf   

        ld      hl,0            ; Do y*12
        ld      d,0
	ex	de,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl
	add	hl,hl

        ld      d,0             ; Do x/8
        ld      e,a
        srl     e
        srl     e
        srl     e
        add     hl,de

        ld      de,$FC00
        add     hl,de           ; Add address to graphbuf

        ld      b,00000111b     ; Get the remainder of x/8
        and     b
        cp	0               ; Is this sprite aligned to 8*n,y?
        jp      z,ALIGN


;   Non aligned sprite blit starts here   

        pop     ix              ; ix->sprite
        ld      d,a             ; d=how many bits to shift each line

        ld      e,8             ; Line loop
LILOP:  ld      b,(ix+0)        ; Get sprite data

        ld      c,0             ; Shift loop
        push    de
SHLOP:  srl     b
        rr      c
        dec     d
        jp      nz,SHLOP
        pop     de

        ld      a,b             ; Write line to graphbuf
        xor     (hl)
        ld      (hl),a
        inc     hl
        ld      a,c
        xor     (hl)
        ld      (hl),a

        ld      bc,15           ; Calculate next line address
        add     hl,bc
        inc     ix              ; Inc spritepointer

        dec     e
        jp      nz,LILOP        ; Next line

        jp      DONE1


;   Aligned sprite blit starts here   

ALIGN:                          ; Blit an aligned sprite to graphbuf
        pop     de              ; de->sprite
        ld      b,8
ALOP1:  ld      a,(de)
        xor     (hl)            ; xor=erase/blit
        ld      (hl),a
        inc     de
        push    bc
        ld      bc,16
        add     hl,bc
        pop     bc
        djnz    ALOP1

DONE1:
        ret

;=================== Greyscale Putsprite routine made by SCaBBy ===================
PutMaskedSprite:
	ld ix,BlankMask
grPutSprite:
	ld d,0
	ld a,c
	add a,a
	add a,a
	rl d
	add a,a
	rl d
	add a,a
	rl d
	ld e,b
	srl e
	srl e
	srl e
	or e
	ld e,a
	ld a,b
	and 7
	inc a
	ld b,a
	dec d
	dec d
	dec d
	dec d
        ld c,16
        ex de,hl
        push hl
        push de
        ld de,GreyBuffer-0FC00h
        add hl,de
        pop de
DoPutMaskedSprite:
	push bc
        ld a,h
        cp 0FCh
        ld a,(ix)
        jr c,NotIncIX
        inc ix
NotIncIX:
	ld c,255
        ex de,hl
	push hl
        ld h,(hl)
	ld l,0
	dec b
	jr z,DontRotateMaskedSprite
RotateMaskedSprite:
	scf
        rra
	rr c
	srl h
	rr l	
	djnz RotateMaskedSprite
DontRotateMaskedSprite:
	ex de,hl
	and (hl)
	xor d
	ld (hl),a
	inc hl
	ld a,c
	and (hl)
	xor e
	ld (hl),a
	ld bc,15
	add hl,bc
        pop de
        inc de
	pop bc
        ex (sp),hl
	dec c
	jr nz,DoPutMaskedSprite
        pop hl
	ret

BlankMask .db 0,0,0,0,0,0,0,0

#include "graphics.inc"

.end
